@@ -24,26 +24,47 @@
 
 Q_LOGGING_CATEGORY(LOG_KSSHASKPASS, "ksshaskpass")
 
-enum Type {
-    TypePassword,
-    TypeClearText,
-    TypeConfirm,
-    TypeConfirmCancel,
+constexpr const char *PROMPT_TYPE_ENV_VAR = "SSH_ASKPASS_PROMPT";
+
+// Standard prompt types defined by openssh.
+enum class PromptType {
+    Confirm,
+    Entry,
+    None,
 };
 
-// Try to understand what we're asked for by parsing the phrase. Unfortunately, sshaskpass interface does not
-// include any saner methods to pass the action or the name of the keyfile. Fortunately, openssh and git
-// has no i18n, so this should work for all languages as long as the string is unchanged.
-static void parsePrompt(const QString &prompt, QString &identifier, bool &ignoreWallet, enum Type &type)
+// Implemented UI display types.
+enum class DisplayType {
+    Password,
+    ClearText,
+    Confirm,
+    ConfirmCancel,
+};
+
+static void parsePrompt(PromptType promptType, const QString &prompt, QString &identifier, bool &ignoreWallet, DisplayType &displayType)
 {
+    if (promptType == PromptType::Confirm) {
+        displayType = DisplayType::Confirm;
+        ignoreWallet = true;
+        return;
+    }
+
+    if (promptType == PromptType::None) {
+        displayType = DisplayType::ConfirmCancel;
+        ignoreWallet = true;
+        return;
+    }
+
+    // "Entry" prompt type: password or text input. We parse several known prompts from openssh and git (which have no i18n)
+    // to extract credential names and determine whether to use cleartext.
     QRegularExpressionMatch match;
 
     // openssh sshconnect2.c
     // Case: password for authentication on remote ssh server
-    match = QRegularExpression(QStringLiteral("^(.*@.*)'s password( \\(JPAKE\\))?: $")).match(prompt);
+    match = QRegularExpression(QStringLiteral("^(.*@.*)'s password: $")).match(prompt);
     if (match.hasMatch()) {
         identifier = match.captured(1);
-        type = TypePassword;
+        displayType = DisplayType::Password;
         ignoreWallet = false;
         return;
     }
@@ -53,7 +74,7 @@ static void parsePrompt(const QString &prompt, QString &identifier, bool &ignore
     match = QRegularExpression(QStringLiteral("^(Enter|Retype) (.*@.*)'s (old|new) password: $")).match(prompt);
     if (match.hasMatch()) {
         identifier = match.captured(2);
-        type = TypePassword;
+        displayType = DisplayType::Password;
         ignoreWallet = true;
         return;
     }
@@ -63,7 +84,7 @@ static void parsePrompt(const QString &prompt, QString &identifier, bool &ignore
     match = QRegularExpression(QStringLiteral("^Enter passphrase for( RSA)? key '(.*)': $")).match(prompt);
     if (match.hasMatch()) {
         identifier = match.captured(2);
-        type = TypePassword;
+        displayType = DisplayType::Password;
         ignoreWallet = false;
         return;
     }
@@ -73,7 +94,7 @@ static void parsePrompt(const QString &prompt, QString &identifier, bool &ignore
     match = QRegularExpression(QStringLiteral("^Enter passphrase for (.*?)( \\(will confirm each use\\))?: $")).match(prompt);
     if (match.hasMatch()) {
         identifier = match.captured(1);
-        type = TypePassword;
+        displayType = DisplayType::Password;
         ignoreWallet = false;
         return;
     }
@@ -84,7 +105,7 @@ static void parsePrompt(const QString &prompt, QString &identifier, bool &ignore
     match = QRegularExpression(QStringLiteral("^Bad passphrase, try again for (.*?)( \\(will confirm each use\\))?: $")).match(prompt);
     if (match.hasMatch()) {
         identifier = match.captured(1);
-        type = TypePassword;
+        displayType = DisplayType::Password;
         ignoreWallet = true;
         return;
     }
@@ -94,103 +115,28 @@ static void parsePrompt(const QString &prompt, QString &identifier, bool &ignore
     match = QRegularExpression(QStringLiteral("Enter PIN for '(.*)': $")).match(prompt);
     if (match.hasMatch()) {
         identifier = match.captured(1);
-        type = TypePassword;
+        displayType = DisplayType::Password;
         ignoreWallet = false;
         return;
     }
 
-    // openssh mux.c
-    match = QRegularExpression(QStringLiteral("^(Allow|Terminate) shared connection to (.*)\\? $")).match(prompt);
-    if (match.hasMatch()) {
-        identifier = match.captured(2);
-        type = TypeConfirm;
-        ignoreWallet = true;
-        return;
-    }
-
-    // openssh mux.c
-    match = QRegularExpression(QStringLiteral("^Open (.* on .*)?$")).match(prompt);
-    if (match.hasMatch()) {
-        identifier = match.captured(1);
-        type = TypeConfirm;
-        ignoreWallet = true;
-        return;
-    }
-
-    // openssh mux.c
-    match = QRegularExpression(QStringLiteral("^Allow forward to (.*:.*)\\? $")).match(prompt);
-    if (match.hasMatch()) {
-        identifier = match.captured(1);
-        type = TypeConfirm;
-        ignoreWallet = true;
-        return;
-    }
-
-    // openssh mux.c
-    match = QRegularExpression(QStringLiteral("^Disable further multiplexing on shared connection to (.*)? $")).match(prompt);
-    if (match.hasMatch()) {
-        identifier = match.captured(1);
-        type = TypeConfirm;
-        ignoreWallet = true;
-        return;
-    }
-
-    // openssh ssh-agent.c
-    match = QRegularExpression(QStringLiteral("^Allow use of key (.*)?\\nKey fingerprint .*\\.$")).match(prompt);
-    if (match.hasMatch()) {
-        identifier = match.captured(1);
-        type = TypeConfirm;
-        ignoreWallet = true;
-        return;
-    }
-
-    // openssh sshconnect.c
-    match = QRegularExpression(QStringLiteral("^Add key (.*) \\(.*\\) to agent\\?$")).match(prompt);
-    if (match.hasMatch()) {
-        identifier = match.captured(1);
-        type = TypeConfirm;
-        ignoreWallet = true;
-        return;
-    }
-
-    // openssh ssh-agent.c
-    // Case: asking to press the button on the security key device
-    // First match group is key type like "ED25519-SK", second is SHA digest of key, like ssh-add -l prints
-    match = QRegularExpression(QStringLiteral("^Confirm user presence for key (.*?) (.*)$")).match(prompt);
-    if (match.hasMatch()) {
-        identifier = match.captured(2);
-        type = TypeConfirmCancel;
-        ignoreWallet = true;
-        return;
-    }
-
     // openssh ssh-agent.c
     // Case: asking to provide the PIN of the security key device
     // match after "for" is key type, match after "key" is SHA digest of key
     match = QRegularExpression(QStringLiteral("^Enter PIN( and confirm user presence)? for (.*?) key (.*?): $")).match(prompt);
     if (match.hasMatch()) {
         identifier = QStringLiteral("PIN:") + match.captured(3);
-        type = TypePassword;
+        displayType = DisplayType::Password;
         ignoreWallet = true;
         return;
     }
 
-    // git imap-send.c
-    // Case: asking for password by git imap-send
-    match = QRegularExpression(QStringLiteral("^Password \\((.*@.*)\\): $")).match(prompt);
-    if (match.hasMatch()) {
-        identifier = match.captured(1);
-        type = TypePassword;
-        ignoreWallet = false;
-        return;
-    }
-
     // git credential.c
     // Case: asking for username by git without specifying any other information
     match = QRegularExpression(QStringLiteral("^Username: $")).match(prompt);
     if (match.hasMatch()) {
         identifier = QString();
-        type = TypeClearText;
+        displayType = DisplayType::ClearText;
         ignoreWallet = true;
         return;
     }
@@ -200,7 +146,7 @@ static void parsePrompt(const QString &prompt, QString &identifier, bool &ignore
     match = QRegularExpression(QStringLiteral("^Password: $")).match(prompt);
     if (match.hasMatch()) {
         identifier = QString();
-        type = TypePassword;
+        displayType = DisplayType::Password;
         ignoreWallet = true;
         return;
     }
@@ -210,7 +156,7 @@ static void parsePrompt(const QString &prompt, QString &identifier, bool &ignore
     match = QRegularExpression(QStringLiteral("^Username for '(.*)': $")).match(prompt);
     if (match.hasMatch()) {
         identifier = match.captured(1);
-        type = TypeClearText;
+        displayType = DisplayType::ClearText;
         ignoreWallet = false;
         return;
     }
@@ -220,7 +166,7 @@ static void parsePrompt(const QString &prompt, QString &identifier, bool &ignore
     match = QRegularExpression(QStringLiteral("^Password for '(.*)': $")).match(prompt);
     if (match.hasMatch()) {
         identifier = match.captured(1);
-        type = TypePassword;
+        displayType = DisplayType::Password;
         ignoreWallet = false;
         return;
     }
@@ -229,7 +175,7 @@ static void parsePrompt(const QString &prompt, QString &identifier, bool &ignore
     match = QRegularExpression(QStringLiteral("^Username for \"(.*?)\"$")).match(prompt);
     if (match.hasMatch()) {
         identifier = match.captured(1);
-        type = TypeClearText;
+        displayType = DisplayType::ClearText;
         ignoreWallet = false;
         return;
     }
@@ -238,16 +184,7 @@ static void parsePrompt(const QString &prompt, QString &identifier, bool &ignore
     match = QRegularExpression(QStringLiteral("^Password for \"(.*?)\"$")).match(prompt);
     if (match.hasMatch()) {
         identifier = match.captured(1);
-        type = TypePassword;
-        ignoreWallet = false;
-        return;
-    }
-
-    // Case: password extraction from mercurial, see bug 380085
-    match = QRegularExpression(QStringLiteral("^(.*?)'s password: $")).match(prompt);
-    if (match.hasMatch()) {
-        identifier = match.captured(1);
-        type = TypePassword;
+        displayType = DisplayType::Password;
         ignoreWallet = false;
         return;
     }
@@ -257,10 +194,10 @@ static void parsePrompt(const QString &prompt, QString &identifier, bool &ignore
     qCWarning(LOG_KSSHASKPASS) << "Unable to parse phrase" << prompt;
 }
 
-void cancelDialog(QWidget *parent, const QString &text, const QString &title)
+void cancelDialog(QWidget *parent, const QString &text)
 {
     QDialog *d = new QDialog(parent);
-    d->setWindowTitle(title);
+    d->setWindowTitle(i18nc("@title:window", "Enter SSH Credentials"));
     d->setObjectName(QStringLiteral("information"));
 
     QDialogButtonBox *buttonBox = new QDialogButtonBox(d);
@@ -276,7 +213,7 @@ int main(int argc, char **argv)
 
     // TODO update it.
     KAboutData about(QStringLiteral("ksshaskpass"),
-                     i18n("Ksshaskpass"),
+                     i18n("SSH Credentials"),
                      QStringLiteral(PROJECT_VERSION),
                      i18n("KDE version of ssh-askpass"),
                      KAboutLicense::GPL,
@@ -297,17 +234,25 @@ int main(int argc, char **argv)
     parser.process(app);
     about.processCommandLine(&parser);
 
+    const QString promptTypeString = qEnvironmentVariable(PROMPT_TYPE_ENV_VAR);
+    PromptType promptType = PromptType::Entry;
+    if (promptTypeString == QLatin1String("confirm")) {
+        promptType = PromptType::Confirm;
+    } else if (promptTypeString == QLatin1String("none")) {
+        promptType = PromptType::None;
+    }
+
     const QString walletFolder = app.applicationName();
     QString dialog = i18n("Please enter passphrase"); // Default dialog text.
     QString identifier;
     QString item;
     bool ignoreWallet = false;
-    enum Type type = TypePassword;
+    DisplayType displayType = DisplayType::Password;
 
     // Parse commandline arguments
     if (!parser.positionalArguments().isEmpty()) {
         dialog = parser.positionalArguments().at(0);
-        parsePrompt(dialog, identifier, ignoreWallet, type);
+        parsePrompt(promptType, dialog, identifier, ignoreWallet, displayType);
     }
 
     // Open KWallet to see if an item was previously stored
@@ -341,16 +286,16 @@ int main(int argc, char **argv)
     }
 
     // Item could not be retrieved from wallet. Open dialog
-    switch (type) {
-    case TypeConfirmCancel: {
-        cancelDialog(nullptr, dialog, i18n("Ksshaskpass"));
+    switch (displayType) {
+    case DisplayType::ConfirmCancel: {
+        cancelDialog(nullptr, dialog);
         // dialog can only be canceled
         return 1;
     }
-    case TypeConfirm: {
+    case DisplayType::Confirm: {
         if (KMessageBox::questionTwoActions(nullptr,
                                             dialog,
-                                            i18n("Ksshaskpass"),
+                                            i18nc("@title:window", "Enter SSH Credentials"),
                                             KGuiItem(i18nc("@action:button", "Accept"), QStringLiteral("dialog-ok")),
                                             KStandardGuiItem::cancel())
             != KMessageBox::PrimaryAction) {
@@ -360,11 +305,11 @@ int main(int argc, char **argv)
         item = QStringLiteral("yes\n");
         break;
     }
-    case TypeClearText:
+    case DisplayType::ClearText:
         // Should use a dialog with visible input, but KPasswordDialog doesn't support that and
         // other available dialog types don't have a "Keep" checkbox.
         /* fallthrough */
-    case TypePassword: {
+    case DisplayType::Password: {
         // create the password dialog, but only show "Enable Keep" button, if the wallet is open
         KPasswordDialog::KPasswordDialogFlag flag(KPasswordDialog::NoFlags);
         if (wallet.get()) {
@@ -373,7 +318,7 @@ int main(int argc, char **argv)
         QPointer<KPasswordDialog> kpd = new KPasswordDialog(nullptr, flag);
 
         kpd->setPrompt(dialog);
-        kpd->setWindowTitle(i18n("Ksshaskpass"));
+        kpd->setWindowTitle(i18nc("@title:window", "Enter SSH Credentials"));
         // We don't want to dump core when the password dialog is shown, because it could contain the entered password.
         // KPasswordDialog::disableCoreDumps() seems to be gone in KDE 4 -- do it manually
         struct rlimit rlim;
